/*
  A program which opens a window and draws the "color cube."

  Use the arrow keys and +/-, PgUp,PgDn, Home,End, Ins,Del 
  keys to make the cube move.

  Use w/W to toggle between wireframe and solid models
  Use c/C to toggle backface culling on/off

  Written by - 
               Parag Chaudhuri
	       Research Scholar,
	       DCSE, IITD.
*/

#include <cstdlib>
//Including only glut.h is sufficient as it includes GL.h and GLX.h for us
#include <GL/glut.h>
#include "colorcube.h"


//Function to initialize some basic parameters
GLvoid InitGL(GLvoid)
{
  //Setting the color used to clear the framebuffer
  glClearColor(0.0, 0.0, 0.0, 1.0f);
  //Setting the depth used to clear the depthbuffer
  glClearDepth(1.0);
  //Enabling Z-buffering
  glEnable(GL_DEPTH_TEST);
  //Enabling Smooth Shading 
  glShadeModel(GL_SMOOTH);
  //Setup backface culling
  glCullFace(GL_BACK);

  //Enable color array
  glEnableClientState (GL_COLOR_ARRAY);
  //Enable vertex array
  glEnableClientState (GL_VERTEX_ARRAY);
  //Specifying data for the color array
  glColorPointer (3, GL_FLOAT, 0, colors); 
  //Specifying data for the vertex array
  glVertexPointer (3, GL_FLOAT, 0, vertices);
}

//-------------------------------------------------------------------------

//GL reshape callback
GLvoid ReshapeGL (GLsizei Width, GLsizei Height)
{
  //Prevent a divison by zero
  if (Height == 0) Height=1;
  //Set the viewport to the whole of the current window
  glViewport (0, 0, Width, Height);
  //Change the matrix mode to projection
  glMatrixMode (GL_PROJECTION);
  //Load the identity matrix
  glLoadIdentity ();
  //Setup a perspective projection
  //Parameters are in the order -
  //vertical field of view angle - 60 degrees
  //Aspect Ratio i.e. width/height of window - 1.0
  //Near clipping plane distance - 0.1
  //Far clipping plane distance - 1000.0
  gluPerspective(45.0f,(GLfloat)Width/(GLfloat)Height, 0.1, 1000.0);
  //Change the matrix mode to modelview
  glMatrixMode (GL_MODELVIEW);
}

//-------------------------------------------------------------------------

//GL display callback - does all the drawing
GLvoid DisplayGL(GLvoid)
{
  //Clear the frame and depth buffers
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  //Setup the camera
  //Camera center at 0.0,5.0,0.0
  //LookAt 0.0,0.0,0.0
  //Up Vector 0.0,1.0,0.0
  gluLookAt(0.0,0.0,10.0,0.0,0.0,0.0,0.0,1.0,0.0);
  //Translate as per the current translation parameters 
  glTranslatef(xpos,ypos,zpos);
  //Rotate as per the current rotation parameters
  glRotatef(xrot,1.0,0.0,0.0);
  glRotatef(yrot,0.0,1.0,0.0);
  glRotatef(zrot,0.0,0.0,1.0);

  //Toggle Backface culling
  if (enable_culling) 
    glEnable(GL_CULL_FACE);
  else
    glDisable(GL_CULL_FACE);

  //Toggle solid/wireframe drawing
  if(solid)
    glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
  else
    glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
  
  //Draw the color cube
  glDrawElements(GL_QUADS, 24, GL_UNSIGNED_BYTE, allIndices);
  
  //Swap the double buffers
  glutSwapBuffers();
}

//-------------------------------------------------------------------------

//GL keyboard callback
GLvoid KeyPressedGL(unsigned char key, GLint x, GLint y) 
{
    switch (key)
      {    
	//quit
        case ESCAPE: exit(1);                   	
	             break;
	//move along Z
        case '+' :zpos+=0.1; 
	          glutPostRedisplay();
		  break;
	//move along Z
        case '-' :zpos-=0.1; 
	          glutPostRedisplay();
		  break;
	//rotate left about Z
        case DELETE:yrot-=1.0;
	            glutPostRedisplay();
		    break;
	//toggle wireframe/solid
        case 'w':
        case 'W':solid = !solid;
	         glutPostRedisplay();
		 break;
	//Toggle culling
        case 'c':
        case 'C':enable_culling = !enable_culling;
	         glutPostRedisplay();
		 break;
         default:
		  break;
      }	

}

//-------------------------------------------------------------------------

//GL Keyboard callback for special keys
GLvoid SpecialKeyPressedGL(GLint key, GLint x, GLint y) 
{
 switch (key)
      {   
       //Move along Y
       case GLUT_KEY_UP:ypos+=0.1; 
	                glutPostRedisplay();
		        break;
       //Move along Y
       case GLUT_KEY_DOWN:ypos-=0.1; 
	                  glutPostRedisplay();
		          break;
       //Move along X
       case GLUT_KEY_LEFT:xpos-=0.1; 
	                glutPostRedisplay();
		        break;
       //Move along X
       case GLUT_KEY_RIGHT:xpos+=0.1; 
	                  glutPostRedisplay();
		          break;
       //Rotate right about Z
       case GLUT_KEY_PAGE_DOWN:yrot+=1.0;
	                    glutPostRedisplay();
			    break;
       //Rotate about X
       case GLUT_KEY_HOME:xrot+=1.0;
	                  glutPostRedisplay();
			  break;
       //Rotate about X
       case GLUT_KEY_END:xrot-=1.0;
	                    glutPostRedisplay();
			    break;
       //Rotate about Y
       case GLUT_KEY_INSERT:zrot+=1.0;
	                    glutPostRedisplay();
			    break;
       //Rotate about Y
       case GLUT_KEY_PAGE_UP:zrot-=1.0;
	                    glutPostRedisplay();
			    break;
       default:
	        break;
      }	
}

//-------------------------------------------------------------------------

// The main rendering function
GLvoid RenderGL(int argc, char** argv)
{
  //Initialize GLUT
  glutInit(&argc, argv);
  //Initialize GLUT display
  glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );
  //Window top-left corner position
  glutInitWindowPosition(0,0 );
  //Window size
  glutInitWindowSize( 500, 500 );
  //Window title
  glutCreateWindow("The Color Cube");

  //Our Init function
  InitGL();

  //Register the callbacks  
  glutDisplayFunc(&DisplayGL);
  glutReshapeFunc(&ReshapeGL);  
  glutKeyboardFunc(&KeyPressedGL);
  glutSpecialFunc(&SpecialKeyPressedGL);
  
  //Start the GLUT event handling loop
  glutMainLoop();
}

//-------------------------------------------------------------------------

int main(int argc, char** argv)
{
  RenderGL(argc,argv);
}

//-------------------------------------------------------------------------

